#include "crystalstructurewidget.h"
#include "ui_crystalstructurewidget.h"
#include "checkboxdelegate.h"
#include "floatdelegate.h"
#include "restraintsdialog.h"
#include "constraintsdialog.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "mainwindow.h"

#include <QStandardItemModel>
#include <QSortFilterProxyModel>
#include <QDebug>

extern "C" void put_crystal_atom_parameter(int idph, int idatom, int idpar, float pvalue);
extern "C" void show_model(int k);

CrystalStructureWidget::CrystalStructureWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::CrystalStructureWidget),
    tableChangedEnabled(true),
    phaseId(-1)
{
    ui->setupUi(this);

    connect(ui->restraintsPushButton, &QPushButton::clicked, this, &CrystalStructureWidget::restraintsPushButtonClicked);
    connect(ui->constraintsPushButton, &QPushButton::clicked, this, &CrystalStructureWidget::constraintsPushButtonClicked);
    connect(ui->showModelPushButton, &QPushButton::clicked, this, &CrystalStructureWidget::showModelPushButtonClicked);

    createAtomsModel();
    createRestraintsModel();
    setAtomsTableView();
}

CrystalStructureWidget::~CrystalStructureWidget()
{
    delete ui;
}

void CrystalStructureWidget::setSymmetry(int phase, double cell[], const QString &spg)
{
    phaseId = phase;
    ui->cellWidget->setCell(cell, spg);
}

void CrystalStructureWidget::setAtoms(const QVector<crydata::AtomProperties> &atoms)
{    
    updateAtomsModel(atoms);

    //Update widgets in atomsTableView
    cbHeader->setChecked(XYZREF_COL, rowCheckBoxClickedCount(XYZREF_COL) == atomsModel->rowCount());
    cbHeader->setChecked(BREF_COL, rowCheckBoxClickedCount(BREF_COL) == atomsModel->rowCount());
    cbHeader->setChecked(OCCREF_COL, rowCheckBoxClickedCount(OCCREF_COL) == atomsModel->rowCount());
}

void CrystalStructureWidget::getSymmetry(double cell[], bool &cellref) const
{
    ui->cellWidget->getCellParameters(cell);
    cellref = ui->cellWidget->isCellRefined();
}

//void CrystalStructureWidget::getAtoms(QVector<AtomProperties> &atoms)
//{
//    for (int r = 0; r < atoms.size(); r++) {

//        QModelIndex index;
//        for (int i = 0; i < 3; i++) {
//            index = atomsModel->index(r, X_COL+i, QModelIndex());
//            atoms[r].xyz[i] = index.data().toFloat();
//        }

//        index = atomsModel->index(r, XYZREF_COL, QModelIndex());
//        for (int rx = 0; rx < 3; rx++) {
//            atoms[r].rcod[rx] = index.data().toInt();
//        }

//        index = atomsModel->index(r, B_COL, QModelIndex());
//        atoms[r].biso = index.data().toFloat();

//        index = atomsModel->index(r, BREF_COL, QModelIndex());
//        atoms[r].rcod[3] = index.data().toInt();

//        index = atomsModel->index(r, OCC_COL, QModelIndex());
//        atoms[r].occ = index.data().toFloat();

//        index = atomsModel->index(r, OCCREF_COL, QModelIndex());
//        atoms[r].rcod[4] = index.data().toInt();
//    }
//}

bool CrystalStructureWidget::hasCellChanged()
{
    return ui->cellWidget->isCellChanged();
}

bool CrystalStructureWidget::hasParameterToRefine()
{
    if (ui->cellWidget->isCellRefined()) return true;
    if (rowCheckBoxClickedCount(XYZREF_COL) > 0) return true;
    if (rowCheckBoxClickedCount(BREF_COL) > 0) return true;
    if (rowCheckBoxClickedCount(OCCREF_COL) > 0) return true;

    return false;
}

void CrystalStructureWidget::createAtomsModel()
{
    atomsModel = new QStandardItemModel(this);
    atomsProxyModel = new QSortFilterProxyModel(this);
    atomsProxyModel->setSourceModel(atomsModel);
    atomsModel->setColumnCount(10);
    const QStringList titles = {"#", "Atom", "x", "y", "z", " ", "Biso", " ", "Occ", " "};
    atomsModel->setHorizontalHeaderLabels(titles);
    connect(atomsModel, &QStandardItemModel::itemChanged, this, &CrystalStructureWidget::atomChanged);
}

void CrystalStructureWidget::createRestraintsModel()
{
    bondRestraintsModel = new QStandardItemModel(this);
    bondRestraintsProxyModel = new QSortFilterProxyModel(this);
    bondRestraintsProxyModel->setSourceModel(bondRestraintsModel);
    bondRestraintsModel->setColumnCount(6);
    const QStringList titles = {"Active", "Type", "Atoms", "Current", "Target", "esd"};
    bondRestraintsModel->setHorizontalHeaderLabels(titles);
}

void CrystalStructureWidget::updateAtomsModel(const QVector<crydata::AtomProperties> &atoms)
{
    tableChangedEnabled = false;
    atomsModel->setRowCount(atoms.size());
    for (int r = 0; r < atoms.size(); r++) {
        auto item = new QStandardItem();
        //item->setForeground(Qt::green); SONO QUI: example to change color
        item->setData(r+1, Qt::DisplayRole);
        item->setTextAlignment(Qt::AlignRight);
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        atomsModel->setItem(r,ID_COL,item);

        item = new QStandardItem(atoms.at(r).label);
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        atomsModel->setItem(r,LABEL_COL,item);

        for (int i = 0; i < 3; i++) {
            item = new QStandardItem();
            item->setData(atoms.at(r).xyz[i], Qt::DisplayRole);
            atomsModel->setItem(r,X_COL+i,item);
        }
        item = new QStandardItem();
        int code = crydata::getAtomCode(atoms.at(r));
        if (code < 0) {
            item->setData(0, Qt::DisplayRole);
            //TOFIX: item disabled but is checkable and white if row is selected.
            //Other strategy: save the disabled checkboxess in a vector in the delegate, use paint to draw disabled checkboxs,
            //TOFIX: disable checkboxheader when all checkboxes are disabled
            item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
            //item->setFlags(item->flags() & ~Qt::ItemIsUserCheckable);
            //item->setData(QVariant(false), Qt::ItemIsEnabled);
        }   else {
            item->setData(code, Qt::DisplayRole);
        }
        atomsModel->setItem(r,XYZREF_COL,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).biso, Qt::DisplayRole);
        atomsModel->setItem(r,B_COL,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).rcod[3], Qt::DisplayRole);
        atomsModel->setItem(r,BREF_COL,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).occ, Qt::DisplayRole);
        atomsModel->setItem(r,OCC_COL,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).rcod[4], Qt::DisplayRole);
        atomsModel->setItem(r,OCCREF_COL,item);
    }

    setItemColor(atoms);

    tableChangedEnabled = true;
}

void CrystalStructureWidget::setAtomsTableView()
{
    ui->atomsTableView->setModel(atomsProxyModel);
    ui->atomsTableView->verticalHeader()->hide();
    ui->atomsTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->atomsTableView->setShowGrid(false);
    ui->atomsTableView->setAlternatingRowColors(true);
    //ui->atomsTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->atomsTableView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    ui->atomsTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->atomsTableView->horizontalHeader()->stretchLastSection();

    auto floatDelegate = new FloatDelegate(ui->atomsTableView);
    floatDelegate->setDecimals(6);
    ui->atomsTableView->setItemDelegateForColumn(X_COL, floatDelegate);
    ui->atomsTableView->setItemDelegateForColumn(Y_COL, floatDelegate);
    ui->atomsTableView->setItemDelegateForColumn(Z_COL, floatDelegate);
    ui->atomsTableView->setItemDelegateForColumn(B_COL, floatDelegate);
    ui->atomsTableView->setItemDelegateForColumn(OCC_COL, floatDelegate);

    auto xyzCheckDelegate = new CheckBoxDelegate(ui->atomsTableView);
    xyzCheckDelegate->setColumn(XYZREF_COL);
    ui->atomsTableView->setItemDelegateForColumn(XYZREF_COL, xyzCheckDelegate);
    connect(xyzCheckDelegate, &CheckBoxDelegate::rowChecked, this,
            &CrystalStructureWidget::checkBoxClicked);

    auto bisoCheckDelegate = new CheckBoxDelegate(ui->atomsTableView);
    bisoCheckDelegate->setColumn(BREF_COL);
    ui->atomsTableView->setItemDelegateForColumn(BREF_COL, bisoCheckDelegate);
    connect(bisoCheckDelegate, &CheckBoxDelegate::rowChecked, this,
            &CrystalStructureWidget::checkBoxClicked);

    auto occCheckDelegate = new CheckBoxDelegate(ui->atomsTableView);
    occCheckDelegate->setColumn(OCCREF_COL);
    ui->atomsTableView->setItemDelegateForColumn(OCCREF_COL, occCheckDelegate);
    connect(occCheckDelegate, &CheckBoxDelegate::rowChecked, this,
            &CrystalStructureWidget::checkBoxClicked);

    cbHeader = new CheckBoxHeader(Qt::Horizontal, {XYZREF_COL, BREF_COL, OCCREF_COL}, false, ui->atomsTableView);
    ui->atomsTableView->setHorizontalHeader(cbHeader);
    cbHeader->setChecked(XYZREF_COL, false);
    cbHeader->setChecked(BREF_COL, false);
    cbHeader->setChecked(OCCREF_COL, false);
    connect(cbHeader, &CheckBoxHeader::checkBoxClicked, this, &CrystalStructureWidget::headerCheckBoxClicked);

    ui->atomsTableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->atomsTableView->setSortingEnabled(true);
    ui->atomsTableView->sortByColumn(0,Qt::AscendingOrder);

    connect(ui->atomsTableView->selectionModel(), &QItemSelectionModel::selectionChanged,
            this, &CrystalStructureWidget::markSelectedAtoms);
}

int CrystalStructureWidget::rowCheckBoxClickedCount(int col) const
{
    int count = 0;
    for (int r = 0; r < atomsModel->rowCount(); r++) {
        QModelIndex index = atomsModel->index(r,col,QModelIndex());
        if (index.data().toBool()) count++;
    }
    return count;
}

bool CrystalStructureWidget::isParInConstraint(int atom, int par)
{
    return (crydata::isParInConstraint(constr, atom, par) >= 0);
}

int CrystalStructureWidget::parInConstraint(int atom, int par)
{
    return crydata::isParInConstraint(constr, atom, par);
}

void CrystalStructureWidget::selectParInConstraint(int atom, int par, int state)
{
    int column;
    if (par < 3 && par >= 0)
        column = XYZREF_COL;
    else if (par == 3)
        column = BREF_COL;
    else if (par == 4)
        column = OCCREF_COL;
    else
        return;

    int con = parInConstraint(atom,par);
    if (con >= 0) {
        for (int i = 0; i < constr[con].atm.size(); i++) {
            if (constr[con].atm[i] != atom) {
                QModelIndex indx = atomsModel->index(constr[con].atm[i], column,QModelIndex());
                atomsModel->setData(indx,state);
            }
        }
    }
}

void CrystalStructureWidget::setItemColor(const QVector<crydata::AtomProperties> &atoms)
{
    for (int r = 0; r < atomsModel->rowCount(); r++) {
        QStandardItem *item;
        for (int i = 0; i < 3; i++) {
            item = atomsModel->item(r,X_COL+i);
            if (atoms.at(r).rcod[i] < 0) {
                item->setForeground(Qt::red);
            } else {
                if (isParInConstraint(r,i))
                    item->setForeground(Qt::blue);
                else
                    item->setForeground(Qt::black);
            }
        }

        item = atomsModel->item(r, B_COL);
        if (isParInConstraint(r,3))
            item->setForeground(Qt::blue);
        else
            item->setForeground(Qt::black);

        item = atomsModel->item(r, OCC_COL);
        if (isParInConstraint(r,4))
            item->setForeground(Qt::blue);
        else
            item->setForeground(Qt::black);
    }
}

void CrystalStructureWidget::setRefineAtomsEnabled(bool enable)
{
//disable all checkboxes in atomsTableView
    for (int r = 0; r < atomsModel->rowCount(); r++) {
        QStandardItem *item;
        item = atomsModel->item(r,XYZREF_COL);
        if (enable) {
            item->setFlags(item->flags() | Qt::ItemIsEnabled);
        } else {
            item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
        }
        // disable title of column: NOT WORKING, TOFIX
        // you might modify checkboxheader.cpp, line 89
        // item = atomsModel->horizontalHeaderItem(XYZREF_COL);
        // if (enable) {
        //     item->setFlags(item->flags() | Qt::ItemIsEnabled);
        // } else {
        //     item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
        // }

        item = atomsModel->item(r,BREF_COL);
        if (enable) {
            item->setFlags(item->flags() | Qt::ItemIsEnabled);
        } else {
            item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
        }

        item = atomsModel->item(r,OCCREF_COL);
        if (enable) {
            item->setFlags(item->flags() | Qt::ItemIsEnabled);
        } else {
            item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
        }
    }
}

void CrystalStructureWidget::headerCheckBoxClicked(bool state, int column)
{
    for (int row = 0; row < atomsModel->rowCount(); row++) {
        QModelIndex index = atomsModel->index(row, column, QModelIndex());
        if (index.flags().testFlags(Qt::ItemIsEnabled))  //TOFIX
            atomsModel->setData(index, QVariant(state?1:0));
    }
}

void CrystalStructureWidget::checkBoxClicked(const QModelIndex &index, bool state)
{
    //Manage parameters in constraint
    if (index.column() == XYZREF_COL) {
        for (int i = 0; i < 3; i++) {
            selectParInConstraint(index.row(),i,state);
        }
    } else if (index.column() == BREF_COL) {
        selectParInConstraint(index.row(),3,state);
    } else if (index.column() == OCCREF_COL) {
        selectParInConstraint(index.row(),4,state);
    }

    //If state is false uncheck the checkBoxHeader
    if (!state && cbHeader->isChecked(index.column())) {
        cbHeader->setChecked(index.column(), false);
        return;
    }

    //If state is true check the checkBoxheader if all checkboxes have been clicked
    if (state && !cbHeader->isChecked(index.column())) {
        if (rowCheckBoxClickedCount(index.column()) == atomsModel->rowCount())
            cbHeader->setChecked(index.column(), true);
    }
}

void CrystalStructureWidget::atomChanged(QStandardItem *item)
{
    if (tableChangedEnabled) {
        if (item->column() == ID_COL || item->column() == LABEL_COL) return;

        put_crystal_atom_parameter(phaseId, item->row()+1, item->column(), item->data(Qt::DisplayRole).toFloat());
    }
}

void CrystalStructureWidget::markSelectedAtoms()
{
    QModelIndexList selection = ui->atomsTableView->selectionModel()->selectedRows();
    QVector<int> selAtoms(selection.count());
    for(int i=0; i < selection.count(); i++)
    {
        selAtoms[i] = atomsProxyModel->mapToSource(selection.at(i)).row();
    }

    MainWindow *mw = getMainWindow();
    if (!mw) return;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = mw->getJav();
#else
    JavWindow *jav = mw->getJav();
#endif
    if (jav->isVisible()) {
        jav->selectAtoms(selAtoms);

        //Raise only if JAV doesn't overlap with SA dialog
        if (!this->window()->geometry().intersects(jav->geometry()))
            jav->raise();
    }
}

void CrystalStructureWidget::restraintsPushButtonClicked()
{
    QStringList labels;
    for (int r = 0; r < atomsModel->rowCount(); r++) {
        auto index = atomsModel->index(r, LABEL_COL);
        labels.append(atomsModel->data(index).toString());
    }

    auto restraintsDlg = new RestraintsDialog(phaseId, labels);
    restraintsDlg->exec();
}

void CrystalStructureWidget::constraintsPushButtonClicked()
{
    auto constraintsDlg = new ConstraintsDialog(phaseId);
    constraintsDlg->exec();
    //SONO QUI: per chiarezza rimuovi rinfo%riding da expo1/2
}

void CrystalStructureWidget::showModelPushButtonClicked()
{
    MainWindow *mw = getMainWindow();
    if (!mw) return;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = mw->getJav();
#else
    JavWindow *jav = mw->getJav();
#endif
    if (!jav->isVisible()) {
        jav->show();
    }
    show_model(phaseId);
}

void CrystalStructureWidget::setConstraints(const QVector<crydata::ConstraintProperties> &newConstr)
{
    constr = newConstr;
}

void CrystalStructureWidget::setRefineEnabled(bool enable)
{
    ui->cellWidget->setRefineEnabled(enable);
    setRefineAtomsEnabled(enable);
}
