#include "restraintsdialog.h"
#include "restraintstargetdialog.h"
#include "ui_restraintsdialog.h"
#include "savedialog.h"
#include "expokeysettings.h"
#include "mainwindow.h"

#include <QSettings>
//#include <QDebug>

extern "C" void read_restraints_file(int idph, const char *file_name, int length, int *ier);
extern "C" void c_put_restraint_target_type(int idph, int target_type);

RestraintsDialog::RestraintsDialog(int phId, const QStringList &labels, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::RestraintsDialog),
    phaseId(phId),
    labAtoms(labels)
{
    ui->setupUi(this);

    distResWidget = new RestraintsWidget(true, this);
    ui->distanceTab->layout()->addWidget(distResWidget);

    angResWidget = new RestraintsWidget(true, this);
    ui->angleTab->layout()->addWidget(angResWidget);

    planResWidget = new RestraintsWidget(false, this);
    ui->planeTab->layout()->addWidget(planResWidget);

    QVector<crydata::RestraintProperties> restr;
    QVector<double> wei(3);    
    crydata::get_restraints_settings(phaseId, wei, targetTypeValue);
    crydata::make_restraints(phaseId, restr, 1);
    setRestraints(restr, labAtoms, wei);

    connect(distResWidget, &RestraintsWidget::restraintsNumberChanged, this, [this](int num){
        ui->distancesLabel->setText(QString::number(num));
    });
    connect(angResWidget, &RestraintsWidget::restraintsNumberChanged, this, [this](int num){
        ui->anglesLabel->setText(QString::number(num));
    });
    connect(planResWidget, &RestraintsWidget::restraintsNumberChanged, this, [this](int num){
        ui->planesLabel->setText(QString::number(num));
    });
    connect(ui->targetPushButton, &QPushButton::clicked, this, &RestraintsDialog::actionTargetButtonClicked);
    connect(ui->savePushButton, &QPushButton::clicked, this, &RestraintsDialog::actionSaveRestraintsClicked);
    connect(ui->loadPushButton, &QPushButton::clicked, this, &RestraintsDialog::actionLoadRestraintsClicked);

    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &RestraintsDialog::buttonBoxClicked);
}

RestraintsDialog::~RestraintsDialog()
{
    delete ui;
}

QVector<crydata::RestraintProperties> RestraintsDialog::getRestraints() const
{
    QVector<crydata::RestraintProperties> restrTot;

    restrTot.append(distResWidget->getRestraints());
    restrTot.append(angResWidget->getRestraints());
    restrTot.append(planResWidget->getRestraints());

    return restrTot;
}

QVector<double> RestraintsDialog::getWeights() const
{
    QVector<double> wei(3);
    distResWidget->getSettings(wei[0]);
    angResWidget->getSettings(wei[1]);
    planResWidget->getSettings(wei[2]);
    return wei;
}

void RestraintsDialog::actionSaveRestraintsClicked()
{
    QSettings settings;
    QString selectedFilter = "All Text Files (*.txt)";
    QString exts = "All Text Files (*.txt);;"
                   "All Files (*)";
    MainWindow *mw = getMainWindow();
    QString curFile = mw->getCurrentFile();
    QString fileName = SaveDialog::run(this,tr("Save Restraints As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(curFile).baseName(),exts,
                                       "txt",selectedFilter);
    if (!fileName.isEmpty()) {
        QFile file(fileName);
        file.open(QFile::WriteOnly | QFile::Text);
        QTextStream stream(&file);
        printRestraints(stream);
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(file).absolutePath());
    }
}

void RestraintsDialog::actionLoadRestraintsClicked()
{
    QSettings settings;
    QString selectedFilter = "All Text Files (*.txt)";
    QString exts = "All Text Files (*.txt);;"
                   "All Files (*)";
    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Load Restraints"), settings.value(DEFAULT_DIR_KEY).toString(),
                                                    exts, &selectedFilter);
    if (!fileName.isEmpty()) {
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());
        int ier;
        read_restraints_file(phaseId, fileName.toStdString().c_str(), fileName.length(), &ier);
        if (!ier) {
            QVector<crydata::RestraintProperties> restr;
            QVector<double> wei(3);            
            crydata::get_restraints_settings(phaseId, wei, targetTypeValue);
            crydata::make_restraints(phaseId, restr, 1);
            setRestraints(restr, labAtoms, wei);
        }
    }
}

void RestraintsDialog::actionTargetButtonClicked()
{
    auto targetDlg = new RestraintsTargetDialog(static_cast<crydata::targetType>(targetTypeValue), this);
    auto ret = targetDlg->exec();
    if (ret == QDialog::Accepted) {
        crydata::targetType newTarget = targetDlg->getTargetType();
        if (targetTypeValue != newTarget) {
            targetTypeValue = newTarget;
            c_put_restraint_target_type(phaseId, targetTypeValue);

            QVector<crydata::RestraintProperties> restr;
            crydata::make_restraints(phaseId, restr, 2);
            QVector<double> wei = getWeights();
            setRestraints(restr, labAtoms, wei);
        }
    }
}

void RestraintsDialog::buttonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if (stdButton == QDialogButtonBox::Ok) {
        QVector<crydata::RestraintProperties> restr = getRestraints();
        QVector<double> wei = getWeights();
        crydata::put_restraints(phaseId, restr, wei);
    } else if(stdButton == QDialogButtonBox::Close) {
        //nothing to do! But you can convert in Cancel and restore parameters
    }
}

void RestraintsDialog::setRestraints(const QVector<crydata::RestraintProperties> &restr, const QStringList &labels, const QVector<double> &wei)
{
    QVector<crydata::RestraintProperties> bondRestr, angRestr, planRestr;
    for (int i = 0; i < restr.size(); i++) {
        if (restr.at(i).code == RESDIST) {
            bondRestr.append(restr.at(i));
        } else if (restr.at(i).code == RESANGLE) {
            angRestr.append(restr.at(i));
        } else if (restr.at(i).code == RESPLANE) {
            planRestr.append(restr.at(i));
        }
    }

    distResWidget->setRestraints(bondRestr, labels, wei.at(0), "distance");
    angResWidget->setRestraints(angRestr, labels, wei.at(1), "angle");
    planResWidget->setRestraints(planRestr, labels, wei.at(2), "plane");
}

void RestraintsDialog::printRestraints(QTextStream &stream)
{
    distResWidget->printRestraints(stream, "weid");
    angResWidget->printRestraints(stream, "weia");
    planResWidget->printRestraints(stream, "weip");
}
